//---------------------------------------------------------------------------------- EEPROM -------------------------------------------------------------------------------------
#include <EEPROM.h>
#define Profile_ADDR 0     // 30 байт
#define Name_ADDR 100      // 40 байт
#define Setting_ADDR 200   // 18 байт 
#define Setting_1_ADDR 300 // 5 байт
#define Init 1023          // Контрольная ячейка
#define InitKey 200        // Контрольное значение

//---------------------------------------------------------------------------------- Подключение дисплея ------------------------------------------------------------------------
#include "Nokia_5110.h"
#define RST 12
#define CE 11
#define DC 10
#define DIN 9
#define CLK 8
Nokia_5110 lcd = Nokia_5110(RST, CE, DC, DIN, CLK);

//----------------------------------------------------------------------------- Подключение регуляторов мощности ----------------------------------------------------------------
#include <GyverTimers.h>                   // Библиотека таймера
#define ZERO_PIN 2                         // Пин детектора нуля
#define INT_NUM 0                          // Соответствующий ему номер прерывания
#define DIM_AMOUNT 2                       // Количество диммеров
const byte dimPins[] = {A3, A2};           // CH1, CH2
int dimmer[DIM_AMOUNT];                    // Переменная диммера
volatile int counter = 0;                  // Счётчик цикла

//----------------------------------------------------------------------------------- Подключение термопар ----------------------------------------------------------------------
#include      <max6675.h>                  // Подключаем библиотеку max6675
const uint8_t DO  = 5;                     // Вывод к которому подключён вывод DO  ( SO, MISO )
const uint8_t CLOCK = 6;                   // Вывод к которому подключён вывод CLK ( SCK, CLOCK )
const uint8_t CS_1  = 3;                   // Вывод к которому подключён вывод CS  ( SS )
const uint8_t CS_2  = 4;                   // Выводы DO и CLK общие для всех модулей, CS для каждого свой
const uint8_t CS_3  = 0;                  
const uint8_t CS_4  = 1;   
//                                         Инициализация модулей
MAX6675       Thermo1(CLOCK, CS_1, DO);    // Датчик платы
MAX6675       Thermo2(CLOCK, CS_2, DO);    // Датчик чипа
MAX6675       Thermo3(CLOCK, CS_3, DO);    // Датчик температуры нижнего нагревателя
MAX6675       Thermo4(CLOCK, CS_4, DO);    // Датчик температуры верхнего нагревателя
int T1,T2,T3,T4;                           // Температура с датчиков

//----------------------------------------------------------------------------------- Настройки профилей и станции ------------------------------------------------------------------------
byte Profile [5][6] = {
  {101,  11, 151, 11, 51, 11},              // 0 - Ручная пайка {Мощность нижнего нагревателя, Мощность верхнего нагревателя, Температура разогрева платы, Температура прогрева чипа, Время разогрева}
  {82,   2, 52,  2, 42, 42},                // 1 - Профиль 1...
  { 73,  153, 253, 13, 43, 53},             // {Температура разогрева платы, Время разогрева платы, Температура прогрева чипа, Время прогрева чипа, Мощность нижнего нагревателя, Мощность верхнего нагревателя}
  {194,   4, 204, 14, 74, 54},              // 3 - Профиль 3 
  {195,   5, 225,  5, 35, 55},              // 4 - Профиль 4
};

char Azbuka[39] = {                                                // Массив букв для названия профиля
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'G', 'K', 'L', 'M',
  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
  ' ', '-', '/', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
};

char Name [4][10] = {                                              // Массив названий профилей
  'P','R','O','F','I','L','E',' ',' ','1',                         // Профиль 1...
  'P','R','O','F','I','L','E',' ',' ','2',
  'P','R','O','F','I','L','E',' ',' ','3',
  'P','R','O','F','I','L','E',' ',' ','4'                          //Профиль 4
};

byte x=0, y=0;                                                     // Порядковые номера изменяемых величин в массиве

int Setting [9] = {                                                // Массив настроек станции
21, 31, 21, 61, 51, 7, 241, 341, 101          
};// 0-Мощность низ макс, 1-Мощность верх макс, 2-Время пайки, 3-Температура вкл кулеров охлаждения, 4-Температура выкл кулеров охлаждения,
  // 5-Время охлаждения до температуры включения кулеров, 6-максимальная температура нижнего нагревателя, 7-максимальная температура верхнего нагревателя, 8-температура включения дымососа

boolean Setting_1 [5]={1,1,1,1,1};         // Автоохлаждение, Контроль 1/2 термопары, Пищалка вкл/выкл, Термопара 3 (низ) вкл/выкл, Термопара 4 (верх) вкл/выкл 

//--------------------------------------------------------------------------------- Переменные для работы меню ------------------------------------------------------------------
byte Page = 0;                             // Страница меню
byte Sel = 0;                              // Изменяемая переменная
byte Sel_Profile = 0;                      // Выбранный профиль в массиве
boolean Shift = false;                     // Флаг возможности изменить переменную
boolean ManHeat = false;                   // Статус нагрева в ручном режиме
//----------------------------------------------------------------------------------- Подключение клавиатуры --------------------------------------------------------------------
#define Key A4                             // Пин клавиатуры (АЦП)
#define Up 383                             // Значение АЦП кнопки вверх
#define Dn 271                             // Значение АЦП кнопки вниз
#define Left 162                           // Значение АЦП кнопки влево
#define Right 820                          // Зачение АЦП кнопки вправо
#define Ok 1023                            // Значение АЦП кнопки ок
boolean U, D, L, R, O;                     // Флаги сработки кнопок
//------------------------------------------------------------------------------------------ Периферия --------------------------------------------------------------------------
#define Cooler A1                          // Вентиляторы охлаждения
#define Light  13                          // Подсветка рабочей зоны
#define Relay  A5                          // Реле защиты
#define Buzzer 7                           // Пищалка
#define Hood   A0                          // Вытяжка

boolean Cool = false;                      // Состояния выходов
boolean Led =  false;
boolean Rel =  false;
boolean Buz =  false;
boolean Fun =  false;

//------------------------------------------------------------------------------------------- Таймеры ----------------------------------------------------------------------------
unsigned long timing;
byte Wait = 100; 
unsigned long TimeSek_A = 0;
unsigned long TimeSek_B = 0;
unsigned long TimeSek_C = 0;
unsigned long Blink=0;        // переменная для отсчёта секунд

//******************************************************************************************* Всякая разная хуйня ************************************************************
  int Temp_A;
  int Seconds_A;
  int TempStart_A=25;
  byte HA=0;
  
  int Temp_B;
  int Seconds_B;
  int TempStart_B=25;
  byte HB=0;

  int Temp_C;
  int Seconds_C;
  int TempStart_C;
  byte HC=0;

  byte SoldProfile = 1; // Выбранный профиль пайки
  byte Stage = 0;       // Этап пайки
  int Sek = 0;          // Время с начала процесса пайки 
  int SolderTime = 0;   // Время пайки чипа для обратного отсчёта

  boolean HeatOn_A = false;
  boolean HeatOn_B = false;
  boolean HeatOn_C = false;
//*******************************************************************************************************************************************************************************  
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
void setup(){   
  pinMode(Cooler, OUTPUT); digitalWrite(Cooler, 0);
  pinMode(Light, OUTPUT);  digitalWrite(Light, 0);
  pinMode(Relay, OUTPUT);  digitalWrite(Relay, 0);
  pinMode(Buzzer, OUTPUT); digitalWrite(Buzzer, 0);  
  pinMode(Hood, OUTPUT);   digitalWrite(Hood, 0);

  pinMode(ZERO_PIN, INPUT_PULLUP);
  for (byte i = 0; i < DIM_AMOUNT; i++) pinMode(dimPins[i], OUTPUT);
  attachInterrupt(INT_NUM, isr, RISING); 
  Timer2.enableISR();
  Timer2.setPeriod(37);
  dimmer[0] = 0;                      // Заданная мощность нагревателя 1
  dimmer[1] = 0;                      // Заданная мощность нагревателя 2

  if (EEPROM.read(Init) != InitKey) { // Если первый запуск устройства
      EE_Clear();                     // Очищаем память
      EE_Writing(5);                  // Записываем начальные значения всех настроек
      EEPROM.write(Init, InitKey);    // Записываем ключ, чтоб в дальнейшем эта функция не вызывалась     
      }  
       
  EE_Read();                          // Читаем настройки из памяти
  Signal(1);
}                                                  
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------                                                     
void loop(){     
  if(HeatOn_A){Heater_A();} else{dimmer[0] = 0;}                          // Нижний нагреватель
  if(HeatOn_B){Heater_B();} else{dimmer[1] = 0;}                          // Верхний нагреватель
  if(HeatOn_C){Heater_C();} else{}                                        // Охлаждение по времени

  if(Setting_1[4]==1){ if(T3>Setting[6]){dimmer[0] = 0;}else{} } else{}   // Контроль максимальной температуры нижнего нагревателя  (если активирован)
  if(Setting_1[5]==1){ if(T4>Setting[7]){dimmer[1] = 0;}else{} } else{}   // Контроль максимальной температуры верхнего нагревателя (если активирован)

  if(T1>Setting[8]||T2>Setting[8]){Fun=1;} else{Fun=0;}                   // Если температура в зоне пайки выше заданной - включаем дымосос

  keys();                                                                 // Опрос кнопок
  Display();                                                              // Отрисовка и управление
  digitalWrite(Cooler, Cool);                                             // Вывод состояния выходов
  digitalWrite(Light, Led);
  digitalWrite(Relay, Rel);
  digitalWrite(Hood, Fun);
  
   if ((millis() - Blink) >= 1000) {                                      // Таймер 1 секунда
    Blink = millis();  
    T1=Thermo1.readCelsius();                                             // Опрос термопар 1 раз в секунду
    if(Setting_1[1]==1){T2=Thermo2.readCelsius();}                        // Если контроль по двум термопарам - читаем Т2
    else{T2=T1;}                                                          // Если контроль по одной Т2=Т1
    if(Setting_1[4]==1){T3=Thermo3.readCelsius();} else{}                 // Если активно ограничение максимальной температуры нижнего нагревателя - читаем Т3
    if(Setting_1[5]==1){T4=Thermo4.readCelsius();} else{}                 // Если активно ограничение максимальной температуры верхнего нагревателя - читаем Т4
    if(HeatOn_A||HeatOn_B||HeatOn_C){Sek++;} else{}                       // Отсчёт общего времени пайки
  }

}    

//-------------------------------------------------------------------------------- Диммеры -------------------------------------------------------------------------------------
void isr() {
  counter = 255;
  Timer2.restart();
}
ISR(TIMER2_A) {
  for (byte i = 0; i < DIM_AMOUNT; i++) {
    if (counter == dimmer[i]){if(dimmer[i]!=0){digitalWrite(dimPins[i], 1);} else{}}               
    else if (counter == dimmer[i] - 1) {digitalWrite(dimPins[i], 0);}  
  }
  counter--;
}

//-------------------------------------------------------------------------------- Пищалка --------------------------------------------------------------------------------------
void Signal(byte repeat){for(uint8_t i=0; i<uint8_t(repeat*2); ++i){ if(Setting_1[2]==1){digitalWrite(Buzzer, !digitalRead(Buzzer));} else{digitalWrite(Buzzer,0);} pause(1);}}

void pause(byte sec){uint32_t new_time = millis(); while(millis() - new_time < uint32_t(sec * 100)){}};

